from __future__ import annotations

from pathlib import Path

from .core_v2 import Loopy


def load(path: str | Path) -> Loopy:
    file_path = Path(path)
    data = file_path.read_text() if file_path.exists() else ""
    return Loopy(data)


def save(tree: Loopy, path: str | Path) -> None:
    file_path = Path(path)
    file_path.write_text(tree.raw)


class FileBackedLoopy(Loopy):
    def __init__(self, path: str | Path) -> None:
        self._path = Path(path)
        data = self._path.read_text() if self._path.exists() else ""
        super().__init__(data, on_mutate=self.sync)

    @property
    def path(self) -> Path:
        return self._path

    def sync(self) -> None:
        self._path.write_text(self.raw)



from loopy import Loopy
from loopy.file_store import FileBackedLoopy, load, save


def test_on_mutate_hook_called():
    calls: list[int] = []

    tree = Loopy(on_mutate=lambda: calls.append(1))
    tree.ls("/")
    assert calls == []

    tree.mkdir("/notes", parents=True)
    assert len(calls) == 1


def test_file_backed_loopy_syncs(tmp_path):
    path = tmp_path / "notes.loopy"
    tree = FileBackedLoopy(path)
    tree.touch("/ideas/mcp", "Expose shell with MCP")

    raw = path.read_text()
    assert "Expose shell with MCP" in raw
    assert "<ideas>" in raw


def test_load_and_save(tmp_path):
    path = tmp_path / "notes.loopy"
    tree = load(path)
    tree.touch("/ideas/cli", "Ship a clean CLI")
    save(tree, path)

    raw = path.read_text()
    assert "Ship a clean CLI" in raw
